/*	EXAMPLE PROGRAM WITH INTERRUPT HANDLING
 *	Copyright (c) 1995 by COSMIC Software
 *
 *	Each received character is copied in a buffer
 *	by the interrupt routine. The main program reads
 *	characters from the buffer and echoes them.
 */
#define _BASE	0
#include <io.h>

#define SIZE	512	/* buffer size */
#define TDRE	0x80	/* transmit ready bit */

/*	authorize interrupts
 */
#define cli()	_asm("cli\n")

/*	Some variables
 */
char buffer[SIZE];	/* reception buffer */
char *ptlec;		/* read pointer */
char *ptecr;		/* write pointer */

/*	Character reception.
 *	Loops until a character is received.
 */
char getch(void)
	{
	char c;			/* character to be returned */

	while (ptlec == ptecr)		/* equal pointers => loop */
		;
	c = *ptlec++;			/* get the received char */
	if (ptlec >= &buffer[SIZE])	/* put in in buffer */
		ptlec = buffer;
	return (c);
	}

/*	Send a char to the ACIA
 */
void outch(char c)
	{
	while (!(SCSR & TDRE))		/* wait for READY */
		;
	SCDR = c;			/* send it */
	}

/*	Character reception routine.
 *	This routine is called on interrupt.
 *	It puts the received char in the buffer.
 */
@port void recept(void)
	{
	*ptecr = SCSR;			/* clear interrupt */
	*ptecr++ = SCDR;		/* get the char */
	if (ptecr >= &buffer[SIZE])	/* put it in buffer */
		ptecr = buffer;
	}

/*	Main program.
 *	Sets up the ACIA and starts an
 *	infinite loop of receive transmit.
 */
void main(void)
	{
	ptecr = ptlec = buffer;		/* initialize pointers */
	BAUD = 0x30;			/* initialize SCI */
	SCCR2 = 0x2c;			/* parameters for interrupt */
	cli();				/* authorize interrupts */
	for (;;)			/* loop */
		outch(getch());		/* get and put a char */
	}

