/***********************************************************/
/*             (C) Barrett Technology Inc.                 */
/*             139 Main Street                             */
/*             Cambridge, MA 02142, USA                    */
/*             Version 4.0        1998                     */
/*             Author: Daniel Griscom                      */
/*             		   Suitable Systems                    */
/*             		   griscom@suitable.com                */
/*             		   www.suitable.com                    */
/***********************************************************/

/* File: arg.h
	This code deals with parsing arguments for a command.
	*/

#ifndef _arg_h
#define _arg_h

/* ====================== Types ======================== */

typedef struct {
	char name[8];		/* Name of parameter */
	int visible;		/* True iff customer should see it */
	unsigned int (*readFunc)(int);	/* Read function; parameter is firstArg */
	void (*writeFunc)(int, unsigned int);	/* Write function: parameters are
					firstArg and value to be written */
	int firstParam;		/* First param to read and write functions */
	char help[52];		/* Help string */
	} argStruct;

/* ====================== Global vars ======================== */

extern const argStruct FArgs[];		/* List of finger [motor] parameters */
extern const argStruct PArgs[];		/* List of global parameters */

/* ====================== Functions ======================== */

/* argIntListParse parses the argument list as a list of integers, which can
	then be retrieved by argIntListGet(). Returns error (if any). */
int argIntListParse(void);

/* argCount returns the number of arguments (or argument pairs in the case
	of argParamPairListParse). */
int argCount(void);

/* argIntListGet returns a given integer argument. whichArg is 1-based. */
unsigned int argIntListGet(int whichArg);

/* argParamListParse takes an argument list structure. It parses the argument
	list, searching for each parameter name. Returns error (if any). */
int argParamListParse(argStruct *argArray);

/* argParamListRead uses the readFunc pointer to return the value of the given
	parameter. Note that the readFunc pointer may take into account the
	current motor num to decide what value to return. */
unsigned int argParamListRead(int whichArg);

/* argParamPairListParse takes an argument list structure. It parse the argument
	list, searching for parameter names and converting values. Returns
	error (if any). */
int argParamPairListParse(argStruct *argArray);

/* argParamPairListWrite uses the writeFunc pointer and the value found in the
	argument list to write the given parameter. Note that the writeFunc
	pointer may take into account the current motor num to decide what
	parameter to write. */
void argParamPairListWrite(int whichArg);

/* argPrintList prints out the arguments in the given argument structure array. If
	showAll is false then only shows visible parameters. */
void argPrintList(argStruct *argArray, int showAll);

/* argFPrintValues prints out the values of the arguments in the finger
	argument structure array. If showAll is false then only shows
	visible parameters. Shows one value for each selected motor. */
void argFPrintValues(int showAll, motorFlags motors);

/* argPPrintValues prints out the values of the arguments in the
	global argument structure array. If showAll is false then only
	shows visible parameters. */
void argPPrintValues(int showAll);

#endif /* #ifdef _arg_h */
