/***********************************************************/
/*             (C) Barrett Technology Inc.                 */
/*             545 Concord Avenue                          */
/*             Cambridge, MA 02138, USA                    */
/*             Version 2.2        1994                     */
/*             Author: Jie Ren                             */
/***********************************************************/
/* bhand.c */

#include <stdio.h>
#include <stdlib.h>
#include "bhand.h"
#include "func.h"
#include "hpdef.h"
/* #include "io.h" */

int Printonoff;      /* Controls serial output: 0 - off, 1 - on*/

ControlType Ctrl;	/* curtmtr, curtfngr, CS, ALE, OE */
ControlType Ctrl2;	/* curtmtr, curtfngr, CS, ALE, OE */
PositionType Pos;	/* union: .plong, .s.lst, ... */
VelocityType Vel;	/* union: .v16, .s.lst, ... */
PositionType PosPC; /* palm start and end positions*/
PositionType PosPO; /* palm start and end positions*/

FilterType Ffilter, Fdefault;  /* finger filter gain, pole, zero ... */
FilterType Pfilter, Pdefault;  /* palm filter gain, pole, zero ... */

TorqueType Mtorque[5];  /* index 1 for motor 1 */
/**
int MotorLimit[5];  
int MotorTorque[5];  
**/

int Phase_store;  /* = 0 */

/* baud rate, .rate - cpu register, .help *100 = actual bps */
const BaudrateType Baud[TOTAL_BAUD] = {   /* table 9-3 */
		{ 0x24, 12},  /* 1200 */
                { 0x23, 24},  /* 2400 */                
                { 0x22, 48},  /* 4800 */
                { 0x21, 96},  /* 9600 */
                { 0x20, 192}, /* 19200 */               
                { 0x01, 384}, /* 38400 */
                { 0x00, 786}, /* ? */
                };

/* Table of commands
   The length of commands (len) has to be in the order of 1, 2 and 3.
   The total number of commands is: Total_Cmd as defined below.
   Make sure that the commands are not overlaped with char 1, 2 and 3.
 */
const CmdType Cmd[] = {
		{1, "R"   , Cmd_quit, " Software Reset."},
		{1, "L"   , Cmd_list, " List all commands available."},
		{2, "HI"  , Cmd_HI, " Hand initialization. Always run HI when there is a problem."},
		{2, "ST",   Cmd_ST, " Stop all motors."},
		{2, "FO",   Cmd_FO, " Open all fingers."},
		{2, "FC",   Cmd_FC, " Close all fingers."},
		{2, "PO",   Cmd_PO, " Palm open (away from the thumb)."},
		{2, "PC",   Cmd_PC, " Palm close (towards to the thumb)."},
		{3, "CLS",  Cmd_greeting, " Clear screen."},
		{3, "HEL",  Cmd_list, " List all commands available."},
		{3, "SFX",  cmdSetMotor, " Select a singer finger."},
		{3, "FSE",  setFingerParameters, " Set finger-motor control parameters."},
		{3, "PSE",  setPalmParameters, " Set palm-motor control parameters."},
		{3, "PSC",  Cmd_PSC, " Palm close by a step."},
		{3, "PSO",  Cmd_PSO, " Palm open  by a step."},
		{3, "1FO",  Cmd_1FO, " Open one finger."},                                                                                                                                
		{3, "1FC",  Cmd_1FC, " Close one finger."},                                                                                                                                
		{3, "DBO",  Cmd_DBO, " Debug: Monitoring HCTL-1100 chips."},                                                                                                                                
		{3, "DBW",  Cmd_DBW, " Debug: Show HCTL-1100 status."},                                                                                                                                
		{3, "SGA",  Cmd_SGA, " Strain gauge reading."},                                                                                                                                

		{3, "FSC",  Cmd_FSC, " Fingers close by a step."},                                                                                                                                
		{3, "FSO",  Cmd_FSO, " Fingers open by a step."},                                                                                                                                		

		/*
                {3, "1SO",  test, " No, Open one finger by a step."},                                                                                                                                                			
                {3, "BAU",  Cmd_baudrate, " Select a baud rate."},
                {3, "PMD",  Cmd_ptest, " Postion Mode."},                
                */
                                                                               
                };
int Total_Cmd; /* =  (sizeof(Cmd) / sizeof(CmdType));*/


void main()
{
	void *eep;
        char ecc;
	VelocityType  VVV;

        clrSerial();		/* initialize serial port */

        /* initHP();	*/	/* software reset HP and init varables */ 
        enableDA();		/* enable port E for DA */

	Ctrl.curtmtr = 0;        /* temp. motor index (varing) */
	Ctrl.curtfngr = 1;       /* select a single finger for 1FO (1-3) */

	Ctrl.curtmtr = 2;        /* BUG !!! */
	Ctrl.curtfngr = 2;       /* select a single finger for 1FO */

	Printonoff = 1;      /* Controls serial output: 0 - off, 1 - on*/
	Total_Cmd =  (sizeof(Cmd) / sizeof(CmdType));
        Cmd_greeting();		/* company information */

        /* 6811 EEPROM Switch (baud rate select) .. */
        /*eep = (void *)BAUDEEprom;**/
	/*eepcpy(eep, "RFirst String", 12);*/
        
        /* eepcpy((void*)BAUDEEprom, "Y", 1);*/
    
        /** ecc = *(char *)eep;  **/
        ecc = *(char *)BAUDEEprom;
	D(printf("\n ecc BAUD = 0x<%02x> ", ecc & 0xFF);)
	/*BAUD = *(char *)BAUDEEprom;*/

	D(printf("\n PORTA = 0x<%02x> ", PORTA & 0xFF);)
        
	/*Cmd_list();
	printf("\n Cmd=%d , %d %d ", Total_Cmd , sizeof(Cmd) , sizeof(CmdType));        
	*/
        initHP();		/* software reset HP and init varables */         
	while (1) {
		int select = parser();
                if ( select < 0 ) {
                	if (select == -1) { D(printf(" Command not found.");) }
                	continue;
                }
		if ( select < 0x1000 ) {
        		D(printf(" Running...");)
        		(*Cmd[select].func)();
	        } else  {
        		printOnoff(" Help: ");
                	printOnoff("%s\n", Cmd[select&0xFFF].help);
         	}
	}
}


