/***********************************************************/
/*             (C) Barrett Technology Inc.                 */
/*             545 Concord Avenue                          */
/*             Cambridge, MA 02138, USA                    */
/*             Version 2.2        1994                     */
/*             Author: Jie Ren                             */
/***********************************************************/
/* func.c */

#include <stdio.h>
#include <stdlib.h>
#include "bhand.h"
#include "hpdef.h"
#include "func.h"

#define BACK_STEP 0x94   /* full open and then back a step */

/* monitor strain gauges */
void Cmd_SGA()
{
        while(1) {
                copReset();
                strainGauge();
                if ( getAch() ) break;
                delay(40); 
        }
}
void Cmd_1FO()
{
        cmdSingOC( Ctrl.curtfngr, 'o');
                slctMotor(Ctrl.curtfngr);        
                cmdTU('u', BACK_STEP);    /* step up/down */
                HP_idle();
        copReset(); 
}
void Cmd_1FC()
{
        cmdSingOC( Ctrl.curtfngr, 'c');
        copReset(); 
}                

void Cmd_quit()
{ 
        printf("  Reseting  \n");
        exit(0);
}

/************/
/* reset and align all motors, HI */
/* First step before executing any commands. */
void Cmd_HI()
{
        int i;

        initHP();	/* software reset HP and init varables */

        for (i=1; i <= 4; i++) {     
                /*copReset();*/
                slctMotor(i);
                resetSetup_HP();                        
                copReset();

                if ( i==4)  setFilter(&Pfilter); /* set default filters */
                else setFilter(&Ffilter);          

                /* alignments */
                HP_write(PROGRAM, 0x01);  /* idle */
                alignCrntMotor();
                /* check status, normal f=0x10 and s=0xC5 */
                /* user can see if HCTL-1100 is running normally */
                D(printf( "\n      Flags = 0x%x ",    HP_read(FLAGS));)
                D(printf( "   Status = 0x%x ", HP_read(STATUS));)                        

                copReset();
                delay(500);                      
        }
        delay(1500);
        Cmd_ST();
}

/* Set filter paparmeters.
   The finger motors have the same filter parameters.
   The palm motor has it own set of filter parameters. 
*/    
void setFilter(FilterType *filter)
{
        copReset();
        HP_write(SAMPLE, filter->sample);
        HP_write(GAINK,  filter->gain);
        HP_write(ZERO,   filter->zero);
        HP_write(POLE,   filter->pole);
        HP_write(MAX_VEL,filter->maxVel);  /* ?? */

        HP_write(ACCEL+1, filter->accel>>8);	/* integer part*/
        HP_write(ACCEL,   filter->accel&0xFF);	/* fraction part*/
}

/* Open/close three fingers at once, 
   finger motors are 1,2 and 3. 
   speed is positive for close 
          negative for open 
   oc = 'o' for open 
   oc = 'c' for close.	   
   proportional velocity mode.
*/
static int fmotOn[4];  /* sych. the three motor for stopping */
void cmdFOC(int oc)
{
        int i;

        fmotOn[1] = 1;
        fmotOn[2] = 1;
        fmotOn[3] = 1;
        D(printf( "\n Move fingers ... ");)

        /* kick off three finger motors*/    
        for (i=1; i <= 3; i++) {       
                copReset();
                slctMotor(i);
                HP_write(PROGRAM, 0x03);      /* control mode */

                if ( oc == 'c' ) Vel.v16 = Ffilter.maxVel;
                else             Vel.v16 = -Ffilter.maxVel;

                D(printf("\n Vel.v16 = 0x%04X  maxVel = 0x%04x", 
                        Vel.v16, Ffilter.maxVel);)

                setProVel();	/* from Vel (structure) */

                HP_write(FLAGS, 0x0B);	     /* set flag F3 and bingo */
        }
        copReset();
        delay(1000);
        copReset();
        while (1) {			    /* wait for finger to stop */
                for (i=1; i <= 3; i++) {       
                        copReset();
                        if ( (!fmotOn[1]) && (!fmotOn[2]) && (!fmotOn[3]) ) return; /*return 1*/
                        slctMotor(i);

                        if ( isVelZero() && fmotOn[i] )  {
                                Mtorque[i].torq = HP_read(PWM);
                                copReset();
                                delay(100);
                                copReset();
                                /* printOnoff( "%x %d-th Torque= 0x%x ",-0x20, i,MotorTorque[i]); */
                                if ( isVelZero())  {   /* double check*/
                                        HP_idle();                         
                                        fmotOn[i] = 0;
                                        /*return 1; */                                
                                }                                
			        copReset();
                        }
                }
                if ( getAch()) break;
                else copReset();
        }
}

/* single motor close/open.
   palm motor is 4 and fingers are 1-3.
   speed is positive for close. 'c' 
          negative for open. 'o' 
   proportional velocity mode 
   motor = 4 : palm move 
   motor = 1-3 : single finger move 
*/   
void cmdSingOC(int motor, int oc)
{
        if ( motor == 4 ) {
                /* set palm velocity */
                if (oc == 'o') Vel.v16 =-0x20; /*{ low=0xE0; high=0xFF;}*/
                else           Vel.v16 = 0x20;  /*{ low=0x20; high=0x00;} */
        } 
        else {	/* finger */
                if (oc == 'c') Vel.v16 = Ffilter.maxVel; /*{ low= Ffilter.maxVel; high=0x00;}*/
                else           Vel.v16 =-Ffilter.maxVel; /*{ low=-Ffilter.maxVel; high=0xFF;}*/
        }

        if ( motor == 4 )   /* palm motor */
        { D(printf( "\n Palm Close/open");) }
        else               /* single finger */
        { D(printf( "\n Single finger move");) }
        D(printf( "... ");)

        /* kick off the selected motor*/
        copReset();
        slctMotor(motor);

        HP_write(PROGRAM, 0x03);      /* control mode */
        setProVel();	/* from Vel (structure) */

        HP_write(FLAGS, 0x0B); 		     /* bingo, flag F3 */
        /* HP_write(FLAGS, 0x0D); */	     /* bingo, flag F5 */    

        copReset();
        delay(1000);
        copReset();
        while (1) {				/* wait for palm to stop */
                copReset();
                if ( isVelZero() )  {
                        delay(100);
                        copReset();
                        if ( isVelZero() )  {   /* double check*/
                                HP_idle();                         
                                break;
                        }                                
                }
                copReset();
                /* any keyhit terminates the move */
                if ( getAch()) { 
                        copReset(); 
                        HP_idle(); 
                        copReset();  
                        break;
                }
        }
        if ( motor == 4 ) {        /* palm move */
                readActPos();
                if ( oc == 'c' ) PosPC.plong = Pos.plong;
        }
        copReset(); 
}

/* current motor step up */
/* trapezoidal moves */
void cmdTU(int ud, int step)
{
        copReset();

        HP_write(PROGRAM, 0x01);    /* enter init/idle mode*/

        HP_clrFlag035();
        readActPos();			/* to Pos (structure) */

        if (ud== 'u')  	Pos.plong += step;
        else        	Pos.plong -= step;        

        setFinPos();		/* from Pos (structure) */

        HP_write(PROGRAM, 0x03);	/* set control mode */
        HP_write(FLAGS, 0x08);		/* set flag 0: start trapezoidal move */


        /**
        while(1) {
                	if ( HP_read(STATUS) & 0x10  ) break;
                        copReset();
        	}  
                ***/              
        copReset();
        delay(100);
        copReset();

        readActPos();			/* to Pos (structure) */

#ifdef DEBUG
        printOnoff( "\n CP=%x %x %x", Pos.s.mst, Pos.s.mid, Pos.s.lst);	
        copReset();
#endif /*DEBUG*/                
}


void setHPparameters(char *ss, FilterType *filter,  FilterType *adefault)
{
        char buff[50];
        copReset();        

        printOnoff( "\n --- Set %s Control Parameters ---", ss); 
        printOnoff( "\n [%s] Set sample time (suggest %d, now=%d):  ", 
	        ss, adefault->sample, filter->sample);
        if (getAstring(buff) ) filter->sample = atoi(buff);
        else printOnoff( "%d ", filter->sample);

        printOnoff( "\n [%s] Set filter gain (suggest %d, now=%d): ", 
        	ss, adefault->gain, filter->gain);
        if (getAstring(buff) ) filter->gain = atoi(buff);        
        else printOnoff( "%d ", filter->gain);

        printOnoff( "\n [%s] Set filter pole (suggest %d, now=%d): ", 
	        ss, adefault->pole, filter->pole);
        if (getAstring(buff) ) filter->pole = atoi(buff);                
        else printOnoff( "%d ", filter->pole);

        printOnoff( "\n [%s] Set filter zero (suggest %d, now=%d): ",
        	ss,  adefault->zero, filter->zero);
        if (getAstring(buff) ) filter->zero = atoi(buff);                
        else printOnoff( "%d ", filter->zero);

        printOnoff( "\n [%s] Set maximum velocity (suggest %d, now=%d): ",
	        ss, adefault->maxVel, filter->maxVel);
        if (getAstring(buff) ) filter->maxVel = atoi(buff);               
        else printOnoff( "%d ", filter->maxVel);

        printOnoff( "\n [%s] Set acceleration (suggest %d, now=%d): ", 
        	ss, adefault->accel, filter->accel);         
        if (getAstring(buff) ) filter->accel = atoi(buff);                                      
        else printOnoff( "%d ", filter->accel);

        printOnoff( "\n [%s] Set increment step (suggest %d, now=%d): ", 
	        ss, adefault->incStep, filter->incStep);
        if (getAstring(buff) ) filter->incStep = atoi(buff);        
        else printOnoff( "%d ", filter->incStep);

        printOnoff( "\n");                
}        

/* set finger fileter parameters */
void setFingerParameters()
{
	int i;
        char buff[30];
        setHPparameters("Finger", &Ffilter, &Fdefault);

        copReset();                        
        for (i=1; i<=3; i++) {
        	slctMotor(i);
	        setFilter(&Ffilter);
	}
}

/* set fileter parameters */
void setPalmParameters()
{
        setHPparameters("Palm", &Pfilter, &Pdefault);
        copReset();                        
                slctMotor(4);
        setFilter(&Pfilter);
}        


/* interactive to select a single finger motor (1-3)*/
void cmdSetMotor()
{
        int idx;
        char buff[50];
        while(1) {
                printOnoff( "\n Select a fingle motor (%d): ", Ctrl.curtfngr);
                if (getAstring(buff) ) idx = atoi(buff);
                copReset();
                if ( idx >= 1 && idx <= 3) { 
                        Ctrl.curtfngr = idx; 
                        break;
                }
                else printOnoff( "\n\t Not a valid selection, try again...");
                copReset();
        }
        /*slctMotor(Ctrl.curtmtr); */
        printOnoff( "\n Finger motor %d selected. ", Ctrl.curtfngr);
}        


/* Make all motors idle: emergency stop */
void Cmd_ST ()
{
        int i;
        for (i = 1; i <= 4; i++)  {    /* Small-C bug, <=4 */
                slctMotor(i);
                HP_idle();
        }
}        


/* fingers up/down (close/open) by a step */
void cmdFUD(int ud)
{
        int i;
        for (i = 1; i <= 3; i++)  {   
                slctMotor(i);        
                cmdTU(ud, Ffilter.incStep);    /* step up/down */
        }        
}


/* palm up/down (close/open) by a step */
void cmdPUD(int ud)
{
        slctMotor(4);
        cmdTU(ud, Pfilter.incStep);
}

/* palm  close/open, palm motor is 4. */
/* speed is positive for close. 'c' */
/*          negative for open. 'o' */
/* proportional velocity mode */
void cmdPOC(int oc)
{
        /* set velocity */
        if (oc != 'c')  Vel.v16 = -0x20;
        else            Vel.v16 = 0x20;

        D(printf( "\n Palm Close/open, press any key ... ");
        )

                /* kick off the palm motor*/
                copReset();
        slctMotor(4);

        HP_write(PROGRAM, 0x03);      /* control mode */
        setProVel();	/* from Vel (structure) */


        HP_write(FLAGS, 0x0B); 		     /* bingo, flag F3 */
        /* HP_write(FLAGS, 0x0D); */	     /* bingo, flag F5 */    

        copReset();
        delay(1000);
        while (1) {				/* wait for palm to stop */
                copReset();
                if ( isVelZero() )  {
                        delay(100);
                        copReset();
                        if ( isVelZero() )  {   /* double check*/
                                HP_idle();                         
                                break;
                        }                                
                }
                /* any keyhit terminates the move */
                if ( getAch()) { 
                        HP_idle(); 
                        break;
                }
                else copReset();
        }
        readActPos();

        if ( oc == 'c' )  PosPC.plong = Pos.plong;
}

void cmdPOCD(int oc)
{
        /* set velocity */
        if (oc != 'o') Vel.v16 = -0x20; /*{ low=0xE0; high=0xFF;}*/
        else           Vel.v16 = 0x20;  /*{ low=0x20; high=0x00;}*/

        D(printf( "\n Palm Close/open, press any key ... ");
        )

                /* kick off the palm motor*/
                copReset();
        slctMotor(4);

        HP_write(PROGRAM, 0x03);      /* control mode */

        setProVel();	/* from Vel (structure) */


        /*
            HP_write(CVELLSB, low);   
            HP_write(CVELMSB, high);
               */

        HP_write(FLAGS, 0x0B); 		     /* bingo, flag F3 */
        /* HP_write(FLAGS, 0x0D); */	     /* bingo, flag F5 */    

        copReset();
        delay(1000);
        while (1) {				/* wait for palm to stop */
                copReset();
                readEncoders();
                if ( isVelZero() )  {
                        delay(100);
                        copReset();
                        if ( isVelZero() )  {   /* double check*/
                                HP_idle();                         
                                break;
                        }                                
                }
                /* any keyhit terminates the move */
                if ( getAch()) { 
                        HP_idle(); 
                        break;
                }
                else copReset();
        }
        readActPos();

        if ( oc == 'c' ) PosPC.plong = Pos.plong;
}

void readEncoders()
{
        int curtmotor;
        int i;

        curtmotor = Ctrl.curtmtr;
        for (i=1; i < 4; i++) {
                Ctrl.curtmtr = i;
                slctMotor(Ctrl.curtmtr);
                copReset();
                readActPos();  /* to Pos (structure) */

                D(printf( " M%d: %6lx ", i, Pos.plong);)
       }

       D(printf( "\n");)
       Ctrl.curtmtr = curtmotor;
}

/* read strain gauges of fingers */
void strainGaugeX() /* BCZ 11/05/99 WAS: void strainGauge() */
{
        int ad1, ad2, ad3, ad4, status;
        int ep1, ep2;
        /*
                        status = peekb(ADCTL);
                        ad1 = peekb(ADR1);
                        ad2 = peekb(ADR2);
                        ad3 = peekb(ADR3);
                        */
        status = ADCTL;
        ad1 = ADR1 & 0xFF;
        /** ad2 = ADR2 & 0xFF;**/ 
        ad2 = ADR4 & 0xFF;    /** bug**/ /* for NGK cpu board only */
        				 /* since the 6811 for that */
                                         /* borad has a damaged pin (46) */
        ad3 = ADR3 & 0xFF;                
        /*
                	        ad4 = peekb(ADR4);                         
                        	ep1 = peekb(0xF801);
        	                ep2 = peekb(0xF802);                                      
        			*/
        /**
	                printOnoff( " status=0x%x \n", 
                        	status);
        			**/
	                                
        printf( "\n Strain Gauges = %2X %2X %2X", ad1,ad2,ad3);

        /**      
        			printOnoff("Status=0x%x a1=%d a2=%d a3=%d a4=%d", 
                	        	status, ad1, ad2, ad3, ad4);
                        	printOnoff( " ep12=%x %x \n", ep1, ep2);
        			**/
        /**
                copReset();                
                	delay(40); 
                        copReset();
                      	if ( getAch() ) {
                       	        copReset();
                        	break;
        		**/

}


void Cmd_FO()  { 
        /*cmdFOC('o'); */
        int i; /* step = 0x64;*/
        copReset();
        cmdFOC('o');
        copReset();         
        for (i = 1; i <= 3; i++)  {   
                slctMotor(i);        
                cmdTU('u', BACK_STEP);    /* step up/down */
                HP_idle();
        }        
}
void Cmd_FC()  { 
        cmdFOC('c'); 
}
void Cmd_PO()  { 
        cmdPOC('o'); 
}
void Cmd_PC()  { 
        cmdPOC('c'); 
}
void Cmd_PDO() { 
        cmdPOCD('o');
}
void Cmd_PDC() { 
        cmdPOCD('c');
}
void Cmd_FSO() { 
        cmdFUD('d'); 
}
void Cmd_FSC() { 
        cmdFUD('u'); 
}
void Cmd_PSC() { 
        cmdPUD('u'); 
}
void Cmd_PSO() { 
        cmdPUD('d'); 
}


/* position monitoring */
void Cmd_DBO()
{ 
        while (1) {
                if ( getAch() ) break;
                copReset();
                moni_position_HP();
        }
}        


/* position mode */
void Cmd_ptest()
{
        int step = 1200;
        int cc;

        Ctrl.curtmtr = Ctrl.curtfngr;
        slctMotor(Ctrl.curtmtr);
        copReset();

        HP_write(PROGRAM, 0x01);    /* enter init/idle mode*/

        HP_clrFlag035();
        readActPos();			/* to Pos (structure) */
        HP_write(PROGRAM, 0x03);	/* set control mode */

        Pos.plong += step;
        setCmdPos();			/* from Pos (structure) */


        copReset();
        delay(100);
        copReset();

        readActPos();			/* to Pos (structure) */

        D(printf( "\n CP=%lx", Pos.plong);
        )
                copReset();

        while (1) {		
                if ( (cc=getAch()) ) {
                        copReset();
                        if ( cc == '-' || cc == '+')  {                        
                                if (cc=='-') Pos.plong--;
                                else  Pos.plong++;                                
                                setCmdPos();			/* from Pos (structure) */

                                D(printf( " %06lx", Pos.plong);)
                        } 
                } 
                else copReset();
                if ( cc == 'q') break;
                else copReset();
        }                
}

/* show flags and other important parameters of
    the currently selected motor.
*/
void Cmd_DBW()
{
	char buf[100];
        printf( "\n HCTL 1100 Parameters\n");

        /*
        sprintf( buf," Flags = %d\n", 1232);
        printf("%s", buf);
	*/
        sprintf(buf, "\n Flags       = 0x%x ", (int) ( HP_read(FLAGS) & 0xFF));
        printf("%s", buf);
        sprintf(buf, "\n STATUS      = 0x%x ", (int) (HP_read(STATUS) & 0xFF));
        printf("%s", buf);
                
        sprintf(buf, "\n COMMANDTor  = 0x%x ", (int) (HP_read(COMMAND) & 0xFF));
        printf("%s", buf);
        sprintf(buf, "\n RING        = 0x%x ", (int) (HP_read(RING) & 0xFF));       
        printf("%s", buf);
        sprintf(buf, "\n X           = 0x%x ", (int) (HP_read(X) & 0xFF));
        printf("%s", buf);
        sprintf(buf, "\n Y           = 0x%x ", (int) (HP_read(Y) & 0xFF));        
        printf("%s", buf);
        sprintf(buf, "\n OFFSET      = 0x%x ", (int) (HP_read(OFFSET) & 0xFF));
        printf("%s", buf);
        sprintf(buf, "\n PHASE       = 0x%x ", (int) (HP_read(PHASE) & 0xFF));                
        printf("%s", buf);
        sprintf(buf, "\n MaxPHASE    = 0x%x ", (int) (HP_read(M_PHASE) & 0xFF));                        
        printf("%s", buf);
	
        sprintf(buf, "\n ZERO        = 0x%x ", (int) (HP_read(ZERO) & 0xFF));
        printf("%s", buf);
        sprintf(buf, "\n POLE        = 0x%x ", (int) (HP_read(POLE) & 0xFF));
        printf("%s", buf);
        sprintf(buf, "\n GAINK       = 0x%x ", (int) (HP_read(GAINK) & 0xFF));
         printf("%s", buf);
       sprintf(buf, "\n ACCEL       = 0x%x ", (int) (HP_read(ACCEL) & 0xFF));
        printf("%s", buf);
        sprintf(buf, "\n MAXVEL      = 0x%x ", (int) (HP_read(MAX_VEL) & 0xFF));             
        printf("%s", buf);
        sprintf(buf, "\n CurMotor    = %d   ", (int) (Ctrl.curtmtr & 0xFF));
        printf("%s", buf);
        
}

