/***********************************************************/
/*             (C) Barrett Technology Inc.                 */
/*             545 Concord Avenue                          */
/*             Cambridge, MA 02138, USA                    */
/*             Version 2.2        1994                     */
/*             Author: Jie Ren                             */
/***********************************************************/
/* hpsetup.c */
#include <stdio.h>
#include "bhand.h"
#include "hpdef.h"

/************************************************************
   Reset controller and setup all flags for position mode.
   Note that SetMotor has to be run first.
 ************************************************************/
void resetSetup_HP()
{
        D(printf( "\n [%d]th HCTL-1100 controller reset ...", Ctrl.curtmtr);)

        HP_clrFlags();

        HP_write(SAMPLE, 0x10);      /* set sample timer for 21*16/1E16 ? */

        HP_write(ACCEL+1, 0x00);     /* write acceleration, integer part */

        HP_write(ACCEL,   0x01);     /* fraction part (/256) */

        HP_write(RING,30);           /* 3 phase motor */
        HP_write(OFFSET, 0x00);	     /* clear offset register*/
        HP_write(PHASE, 0x00);       /* Phase_store*/
        HP_write(M_PHASE, 0x30);        
        HP_write(Y, 5);	             /* set up X and Y for overlaps */
        HP_write(X, 5);
        HP_write(STATUS, 0x05);      /* 5: select 3 phase, single mode commutation*/

        D(printf( " Done! ");)
}        

/* softreset of HCTL-100 and the global parameters */
void initHP() 
{       
	int i; 
        Phase_store = 0;
        Pos.s.mst = 0;
        Pos.s.mid = 0;
        Pos.s.lst = 0;

        for (i=1; i <=5; i++) {   /* index start with 1 form motor 1 */
        	Mtorque[i].limit = 0;
                Mtorque[i].torq = 0xff;
        }
        
        /**
        MotorLimit[0] = 0;   
        MotorLimit[1] = 0;
        MotorLimit[2] = 0;
        MotorLimit[3] = 0;                        
        MotorLimit[4] = 0;
	**/
        
	for (i=1; i <=4; i++) {
        	slctMotor(i);
	        HP_write(PROGRAM, 0x0);    /* software reset */
        	/*HP_write(PROGRAM, 0x01);  auto enter init/idel mode after software reset */
	}
        
        /* fingers */
        Ffilter.gain = 0x41;		/* TL-1100 parameters */
        Ffilter.accel = 0x01;
        Ffilter.pole  = 0x42;
        Ffilter.zero  = 221;
        Ffilter.maxVel = 100;
        Ffilter.sample = 0x10;		
        Ffilter.incStep = 1200;		/* motor step up/dn size */
        memcpy( &Fdefault, &Ffilter, sizeof(FilterType));

        /* palm */
        Pfilter.gain  = 0x64;		/* TL-1100 parameters */
        Pfilter.accel = 0x01;
        Pfilter.pole  = 0x42;
        Pfilter.zero  = 221;
        Pfilter.maxVel = 0x43;
        Pfilter.sample = 0x10;		
        Pfilter.incStep = 200;		/* motor step up/dn size */
        memcpy( &Pdefault, &Pfilter, sizeof(FilterType));
}

#ifdef LATER
/***************************************************
  print out the status and flags of the HCTL-1100
 **************************************************/
status_Out_HP()
{
        int a;

        D(printf( "\n Status : ");)

        a = HP_read(STATUS);
        D(printf( "\n HP Status: 0x%x ", a);)

                a = HP_read(FLAGS);
        D(printf( "\n HP Flags:  0x%x", a);)
}
#endif /*LATER*/


        /********************************************** 
              when the actual speed is zero return 1
                    otherwise            return 0;
         **********************************************/            
int isVelZero()
{
        if ( HP_read(VELMSB) != 0 ) return 0;
        if ( HP_read(VELLSB) != 0 ) return 0;
        return 1;
}	

