/***********************************************************/
/*             (C) Barrett Technology Inc.                 */
/*             139 Main Street                             */
/*             Cambridge, MA 02142, USA                    */
/*             Version 4.0        1998                     */
/*             Author: Daniel Griscom                      */
/*             		   Suitable Systems                    */
/*             		   griscom@suitable.com                */
/*             		   www.suitable.com                    */
/***********************************************************/
/* File: main.c
	This is the top level code for the hand. */

#include "bhand.h"
#include "hp.h"
#include "main.h"
#include "timer.h"
#include "utils.h"
#include "cmd.h"
#include "parse.h"
#include "serial.h"
#include "line.h"
#include "temp.h"
#include "param.h"

/* ================= Prototypes =============== */

void main1(void);
void main2(void);
void testHCTLFlags(void);

/* ================== Main routine ============== */

#ifdef __MWERKS__
int main(void)
#else
void main(void)
#endif /* ifdef __MWERKS__ */
{
	char *firstWord;	/* First word in command line */
	CommandType *theCmd;	/* Parsed command */
	int err;		/* Error (if any) returned by command */
	int motor;

	intDisable();
	timerInit();
	serInit();
	lineInit();
	tempInit();
	paramInit();
	hpReset();
	strainGaugeInit();
	intEnable();

	CmdVersion();

	/* Main command loop: passes through loop once per command */
	while (true) {
		/* Output a prompt */
		printf("\n=> ");

		/* Line collecting loop */
		while (true) {
			copReset();
			/* Collect characters into a line */
			lineCollect();

			/* If any abort chars have been received... */
			if (serAbortWaiting()) {
				/* Clear line, any other chars, and go to next prompt */
				lineClear();
				serAbortClear();
				break;
			}
			/* If a complete line has not been collected... */
			if (!lineDone())
				continue;	/* then continue inner loop. */

			/* Line collected [will not continue inner loop from here]. Parse it. */
			D2(printf("\n  Line done."));
			/* If line is empty, go to next prompt */
			if (lineWordCount() == 0) {
				D2(printf("... but it was empty."));
				lineClear();
				break;
			}

			/* Get pointer to first word in command line */
			firstWord = lineWordGet(0);

			/* Parse word. If fail... */
			if (!parseIt(firstWord)) {
				printf("\nERR %d", errBadCommand);
				lineClear();
				break;
			}

			/* Parsing succeeded. */
			theCmd = parseCmd();
			/* Check if is help command */
			if (parseWasHelp()) {
				/* !!!! Test this: should respond to "?", "?<cmd>" and "?A" [for "list all"] */
				/* If matched with empty cmd, then list commands. */
				if (theCmd->name[0] == '\0') {
					(theCmd->func)();
					lineClear();
					break;
				}
				/* Matched with non-empty command: print command's help */
				printf("\n%s: %s", theCmd->name, theCmd->help);
				lineClear();
				break;
			/* Check if not help, but matched empty command */
			} else if (theCmd->name[0] == '\0') {
				printf("\nERR %d", errBadCommand);
				lineClear();
				break;
			}

			/* Check if had motor prefixes, but wasn't motor cmd */
			if (!theCmd->isMotor && parseWasMotor()) {
				printf("\nERR %d", errNoMotorPrefix);
				lineClear();
				break;
			}
			/* Now run the damn thing. */
			D2(printf("\nOK: about to run command %s.\n", theCmd->name));
		/*	printf("\n**************** Before command **********");
			for (motor = 1; motor <= 4; motor++) {
				hpSetMotor(motor);
				hpPrintState();
			}*/
			err = (theCmd->func)();
		/*	printf("\n**************** After command **********");
			for (motor = 1; motor <= 4; motor++) {
				hpSetMotor(motor);
				hpPrintState();
			}*/
			if (err != errNone)
				printf("\nERR %d", err);
			lineClear();
			serAbortClear();
			break;
		}
	}
}

/* ================== Test routines ============== */

// This tests for problems reading and writing Motor 4's HCTl-1100's flags register.
void testHCTLFlags(void)
{
	int loop, flag, state;
	printf("\ntestHCTLFlags starting...");
	for (loop = 0; loop < 20; loop++) {
		printf("\n***** Loop number %x *****", loop);
		for (flag = 0; flag <= 5; flag++)
			if (flag != 1) {		// Can't set or clear init/idle flag
				printf("\nWorking with flag %x; flags = %2.2x", flag, hpRegRead(FLAGS));
				for (state = 8; state >= 0; state -= 8) {
					hpRegWrite(FLAGS, flag + state);
					if (state)
						printf("\n    Set: flags = %2.2x", hpRegRead(FLAGS));
					else
						printf("\n    Cleared: flags = %2.2x", hpRegRead(FLAGS));
				}
			}
	}
}

void main2(void)
{
	int i;
	int j = 1;

	intDisable();
	timerInit();
	serInit();
	intEnable();

	while (true) {
		printf("\nStarting one second time:");
		for (i = 1; i <= 100; i++)
			timerTickWait(1);
		printf(" done (loop %x)", j++);
	}
}

/* This routine detected an obscure RAM addressing bug in Clive's PLD. */
void main1(void)
{
	volatile int *p, *p1;
	volatile int n;

	intDisable();
	timerInit();
	serInit();
	intEnable();

	D(printf("We're about to start the test here...\n"));

	for (p = (int *)0x3400; p < (int *)0x4000; p = (int *)((char *)p + 1)) {
		p1 = p;
		n = (((unsigned int)p) % 0x100) << 8;
		*p = n;
		D(printf(" %4x:%4x/%4x", p, n, *p1));
		if (*p1 != n)
			D(printf("\PanIc! Stored %4x, read %4x!", n, *p));
	}
	D(printf("\nWell, that seemed to work."));
	while (true)
		;
}

