/***********************************************************/
/*             (C) Barrett Technology Inc.                 */
/*             139 Main Street                             */
/*             Cambridge, MA 02142, USA                    */
/*             Version 4.0        1998                     */
/*             Author: Daniel Griscom                      */
/*             		   Suitable Systems                    */
/*             		   griscom@suitable.com                */
/*             		   www.suitable.com                    */
/***********************************************************/

/* File: motor.h
	This code contains the actual motor control loops. */

#ifndef _motor_h
#define _motor_h

#include "hp.h"

#define kOpen 'o'
#define kClose 'c'

/* =================== Main Functions ====================== */

/* motorStopOne stops one motor. If it's the palm, or if parameter
	FMOTOR is true, leaves the power on and the motor in
	position-holding mode. Otherwise turns off motor power. */
void motorStopOne(int motor);

/* motorTerminate turns off all/given motor power, regardless of FMOTOR. */
int motorTerminate(motorFlags motors);

/* motorMoveOneToLimit takes a motor number and a direction, and drives
	the given motor to the limit. Returns error. */
int motorMoveOneToLimit(int motor, int direction);

/* motorMoveToLimit takes a motorFlags structure and a direction, and
	drives the given motors to the limit. Returns error. */
int motorMoveToLimit(motorFlags motors, int direction);

/* motorMoveToPos takes a motorFlags and a position array, and
	moves the given motors to the position. Returns error. */
int motorMoveToPos(motorFlags motors, PositionType posArray[4 + 1]);

/* motorMoveRel takes a motorFlags, a direction and a distance, and
	moves the given motors the given distance. Returns error. */
int motorMoveRel(motorFlags motors, int direction, unsigned int distArray[4 +1]);

/* motorPositionGet returns the given motor's position as an unsigned integer. */
unsigned int motorPositionGet(int motor);
	

#endif /* #ifndef _motor_h */
