/***********************************************************/
/*             (C) Barrett Technology Inc.                 */
/*             139 Main Street                             */
/*             Cambridge, MA 02142, USA                    */
/*             Version 4.0        1998                     */
/*             Author: Daniel Griscom                      */
/*             		   Suitable Systems                    */
/*             		   griscom@suitable.com                */
/*             		   www.suitable.com                    */
/***********************************************************/

/* File: parse.h
	This code deals with parsing the command word in the command line.
	First, call parseIt with a pointer to the word. Then you can
	call the various access routines to get the results of the
	parsing.

	This code decodes any help or motor prefix in a command. The
	possible prefixes are:

	? - Help: help is printed for this command
	1, 2, 3, 4 - command acts on motor 1, 2, 3 or 4, respectively
	S - command acts on finger spread (motor 4)
	G - command acts on finger grasp (motors 1, 2 and 3)
	*/

#ifndef _parse_h
#define _parse_h

#include "cmd.h"
#include "bhand.h"
#include "hp.h"

/* parseIt takes a pointer to the null-terminated first word in the
	command line. Parses word for help or prefix characters, and
	searches for a command with the given name. Returns true iff
	successful. */
int parseIt(char *firstWord);

/* parseCmd returns a pointer to the command structure for the parsed
	command. Returns null if not found. */
CommandType *parseCmd(void);

/* parseWasHelp returns true iff there was a help character. */
int parseWasHelp(void);

/* parseWasMotor returns true iff there were any motor selection characters. */
int parseWasMotor(void);

/* parseGetMotors sets the given motorFlags structure to the set of motors
	most recently parsed. */
void parseGetMotors(motorFlags motors);

#endif /* #ifndef _parse_h */
