/***********************************************************/
/*             (C) Barrett Technology Inc.                 */
/*             545 Concord Avenue                          */
/*             Cambridge, MA 02138, USA                    */
/*             Version 2.21        1995                    */
/*             Author: Jie Ren                             */
/***********************************************************/
/* parser.c */

#include <stdio.h>
#include <string.h>
#include "bhand.h"
#include "hpdef.h"

static const char prompt[] = "Hand-> ";
static const char prmptChar[] = "*1234";


/********************************************************
   parse the user inputs.
   returns an index of command with an offset.
   offset = 0		to run a command
   offset = 0x1000	to show the help for a command

	return  -2		List  (?CR)
        return  -1		No match
        return -13		CR or *CR

        return   k		k-th command
        return   k+0x1000	k-th help  (?cmd)
********************************************************/
int parser()
{
        int helpoffset = 0;
        int i, ccc[4];
        /*int Ctrl.curtfngr  = 1;*/

        printf( "\n[%c]", prmptChar[Ctrl.curtfngr]);
        printf( "%s", prompt);     

        /* check the first character */
        ccc[0] = getAchar();                     /* get the first character */
        if ( ccc[0] == '?' ) {
                ccc[0] = getAchar();
                if ( ccc[0] == CR ) return -2;       /* ?CR -> listcommands*/
                else helpoffset = 0x1000;            /* ?cmd */
        } 
        else  if ( ccc[0] == CR ) return -13;  /* CR */

        for (i=0; i < Total_Cmd; i++) {
                if ( Cmd[i].len != 1 ) break;
                if ( Cmd[i].cmd[0] == ccc[0]) return (helpoffset+i);
        }
        /* check two-character commands */
        ccc[1] = getAchar();
        if (ccc[1] == CR ) return -1;
        for ( ; i < Total_Cmd; i++) {
                if ( Cmd[i].len != 2 ) break;
                if ( (Cmd[i].cmd[0]==ccc[0]) && (Cmd[i].cmd[1]==ccc[1])) 
                        return (helpoffset+i);
        }

        /* check three-character commands */
        ccc[2] = getAchar();
        if (ccc[2] == CR ) return -1;
        for ( ; i < Total_Cmd; i++) {
                if ( Cmd[i].len != 3 ) break;
                if ( (Cmd[i].cmd[0]==ccc[0]) && (Cmd[i].cmd[1]==ccc[1])
                    && (Cmd[i].cmd[2]==ccc[2])) 
                        return (helpoffset+i);
        }

        return -1;		/* no match */
}


/***********************************************************
  list on screen all the commands available 
***********************************************************/
#define NUM_LINE  6     /* number of commands per line */
void Cmd_list()
{
        int i,j, index;

        clear_screen(10);        
        printOnoff( "\n\t========= Commands Available ===========");
        for (i = 0; i < Total_Cmd; i++) {
                if ( (i%NUM_LINE) == 0 ) printOnoff( "\n\tCMD:   ");
                printOnoff( "%-3s   ", Cmd[i].cmd);
                /* copReset(); */           
        }
        printOnoff( "\n\t======================================== ");

        printOnoff("\n\n\t  ! Start : run `HI' first !");	    
        printOnoff(  "\n\t  ! Finish: run `FC' before ");
        printOnoff(  "\n\t  !         turning power off!");
        printOnoff("\n\n\t  ! Help:   ?* ");	
        printOnoff(  "\n\t  !         where `*' is a command!");
        printOnoff(  "\n\t  No carriage return is needed after commands.");    
        printOnoff(  "\n\n");	    
}

/* clear screen */
void clear_screen(int row)
{
        int i;
        for (i=0; i < row; i++) printOnoff("%c",'\n');
}

/**************************************************************
 copyright greetings
**************************************************************/
void Cmd_greeting()
{
        clear_screen(11);
        printOnoff( "\n\t\t BH8-200 Hand Interactive Commands");
        printOnoff( "\n\t\t          Version 2.1             ");
        printOnoff( "\n\t\t           March 1995             \n");
        printOnoff( "\n\t\t     Barrett Technology Inc.      ");
        printOnoff( "\n\t\t        139 Main Street           ");
        printOnoff( "\n\t\t     Cambridge, MA 02142, USA     ");
        printOnoff( "\n\t\t       Tel: 617-252-9000          ");
        printOnoff( "\n\t\t       Fax: 617-252-9021          ");
        /* printOnoff( "\n\t\t       Email: robot@barrett.com");*/
        printOnoff( "\n\n\n\n\n\n");
}



void Cmd_baudrate()
{
        int i = -1, c;
        int currentBaud = 0;

        c = readBaudrate();
        if ( c >= 0 )  currentBaud = Baud[c].dhlp;

        while ( i < 0 || i > TOTAL_BAUD) {
                printf("\n\t Current Baud rate is %d00 bps.", currentBaud);
                printf("\n\t Baud rates available: ");
                for ( i=0; i < TOTAL_BAUD; i++) {
                        printf("\n\t  [%d]: Baud rate %d00", i, Baud[i].dhlp);
                }
                printf("\n\t  [%d]: Quit", TOTAL_BAUD);
                printf("\n\t -------------------");
                printf("\n\t Your choice: ");
                i = getAchar() - '0';
        }
        if ( i != TOTAL_BAUD) {
                printf("\n\t You selected baud rate of %d00 bps", Baud[i].dhlp);
                printf("\n\t  Are you sure (Y/N)? ");
                c = getAchar();
        }
        if ( (c=='Y') || (c=='y') ) {
                BAUD = Baud[i].rate; 	/* set new baud rate */
                eepcpy((void*)BAUDEEprom, &Baud[i].rate, 1);
                printf("\n\t Baud rate has been set to %d00 bps.", Baud[i].dhlp);
        } 
        else {
                printf("\n\t Baud rate %d00 ramines unchanged.", currentBaud);
        }
}        
