/***********************************************************/
/*             (C) Barrett Technology Inc.                 */
/*             139 Main Street                             */
/*             Cambridge, MA 02142, USA                    */
/*             Version 4.0        1998                     */
/*             Author: Daniel Griscom                      */
/*             		   Suitable Systems                    */
/*             		   griscom@suitable.com                */
/*             		   www.suitable.com                    */
/***********************************************************/

/* File: serial.h
	This code deals with interrupt-driven serial input. 
	Note that Control-C (0x03) characters abort any command
	or input. */

#ifndef _serial_h
#define _serial_h

/* serInit initializes the serial code. Assumes interrupts are disabled. Doesn't
	set baud rate. */
void serInit(void);

/* serBaudSet sets the serial port baud rate to theRate*100. Returns true iff
	it succeeded. */
int serBaudSet(unsigned int theRate);

/* serAbortWaiting returns the number of abort characters in the input buffer. */
int serAbortWaiting(void);

/* serAbortClear flushes the input buffer up to and including the most recent
	abort characters. If no abort characters, then does nothing. */
void serAbortClear(void);

/* serCharsWaiting returns the number of characters in the input buffer (unless
	any of them are abort characters, in which case it returns 0). */
int serCharsWaiting(void);

/* serCharGet waits for a character from the input buffer. It takes the character,
	upper-cases it, calls serCharPut with it, and returns it. */
int serCharGet(void);

/* serCharGetRaw waits for a character from the input buffer and then returns it. */
int serCharGetRaw(void);

/* serSByteGet gets a single signed byte from the input buffer. Waits until
	a byte is available. Ignores abort characters. */
int serSByteGet(void);

/* serUByteGet gets a single unsigned byte from the input buffer. Waits until
	a byte is available.  Ignores abort characters. */
int serUByteGet(void);

/* serWordPut sends a 16-bit word as two bytes. Waits until the second byte is 
	being sent. */
void serWordPut(int w);

/* serCharPut sends a character. */
void serCharPut(int c);

#endif /* #ifndef _serial_h */
