/***********************************************************/
/*             (C) Barrett Technology Inc.                 */
/*             139 Main Street                             */
/*             Cambridge, MA 02142, USA                    */
/*             Version 4.0        1998                     */
/*             Author: Daniel Griscom                      */
/*             		   Suitable Systems                    */
/*             		   griscom@suitable.com                */
/*             		   www.suitable.com                    */
/***********************************************************/

/* File: timer.h
	This code deals with the tick timer and various timing
	routines. It is centered around a timer structure, which
	defines an interval by holding a start time and an end time.
	The code uses Output Compare 4 to run a 10ms timer. The
	timer overflows every 11 minutes. The code assumes that
	the bottom 2 bits of TMSK2 are set to binary 01 giving
	a divide-by-4 prescaler for thetimer, and that the CPU
	runs on a 4.9152 MHz crystal (the clock given to the
	prescaler is the crystal divided by 4). */

#ifndef _timer_h
#define _timer_h

typedef struct {
	unsigned int startTime;
	unsigned int endTime;
	} timer;

/* ticksPerSec is the number of ticks per second (100). */
#define ticksPerSec ((unsigned int)100)

/* timerInit initializes the timer code. It assumes that interrupts are
	disabled. */
void timerInit(void);

/* timerGetTicks returns the current time (in ticks). */
unsigned int timerGetTicks(void);

/* timerOver takes a timer and returns true iff the current time
	is outside the structure's defined time. */
int timerOver(timer *theTimer);

/* timerSet sets a timer to start now and end howLong ticks in the
	future. */
void timerSet(timer *theTimer, unsigned int howLong);

/* timerTickWait waits a given number of ticks. */
void timerTickWait(unsigned int howLong);

/* timerSecWait waits a given number of seconds. Doesn't work for more
	than about 10 minutes (600 seconds). */
void timerSecWait(unsigned int howLong);

/* timerMSWait waits a given number of milliseconds. Uses the Output Compare 3. */
void timerMSWait(unsigned int howLong);

#endif /* ifndef _timer_h */
