/***********************************************************/
/*             (C) Barrett Technology Inc.                 */
/*             139 Main Street                             */
/*             Cambridge, MA 02142, USA                    */
/*             Version 4.0        1998                     */
/*             Author: Daniel Griscom                      */
/*             		   Suitable Systems                    */
/*             		   griscom@suitable.com                */
/*             		   www.suitable.com                    */
/***********************************************************/

/* File: utils.c
	This holds various utilities. */

#include "bhand.h"
#include "hp.h"
#include "param.h"
#include "motor.h"
#include "loop.h"
#include <io.h>
#include "utils.h"

/* eeCopy copies data into the EEPROM, from *s2 to *s1. Calls copReset
	regularly. */
void eeCopy(void *s1, const void *s2, unsigned int n)
{
	int x;
	char *sp1;
	const char *sp2;
	int intLevel;

	sp1 = s1;
	sp2 = s2;
	intLevel = intGet();
	
	D2(printf("\neeCopy to %4.4x from %4.4x; %d bytes.", sp1, sp2, n));

	for (x = n; x > 0; x-= sizeof(char)) {
		D2(printf("\n    copying to %4.4x from %4.4x...", sp1, sp2));
		intDisable();
		eepcpy(sp1, sp2, sizeof(char));
		intSet(intLevel);
		sp1++;
		sp2++;
		copReset();
	}
}

/* eeVecSet writes an EEPROM interrupt vector. */
void eeVecSet(void *vecLoc, voidFuncVoidPtr funcLoc)
{
	eeCopy(vecLoc, (void *)&funcLoc, 2);
}

/* copReset resets the Computer Operating Properly timer. */
void copReset(void)
{
	COPRST = 0x55;
	COPRST = 0xAA;
}

/* strainGaugeInit initializes the strain gauge conversion. */
void strainGaugeInit(void)
{
	OPTION |= 0x80;	/* Turn on A/D charge pump (must be done
				first) */

	ADCTL = 0xB4;	/* Scan top four A/D inputs continuously into
				four A/D registers. */
}

/* strainGauge returns the given finger's strain gauge reading. */
int strainGauge(int motor)
{
	int result;
	/* Read proper A/D conversion register */
	if (motor == 1)
		result = (ADR1 & 0xFF);
	else if (motor == 2)
		result = (ADR2 & 0xFF);
	else if (motor == 3)
		result = (ADR3 & 0xFF);
	else
		result = (ADR4 & 0xFF);	/* This is for the palm; not used */
	/* Flip results? */
	/* DTG, 5/11/98: Changing sense, so that value is flipped if kFParamSGFLIP is FALSE... */
	if (!paramFRead(motor, kFParamSGFLIP))
		result = 0xFF - result;
	return result;
}

/* strainGaugeRead returns the current finger's strain gauge reading. It ignores
	its argument. */
unsigned int strainGaugeRead(int garbage)
{
	return (unsigned int)strainGauge(hpGetMotor());
}

/* positionRead gets the current motor's position, saves it in the lastReportedPos[] array
	(for loop.c's use), and returns it. Ignores its argument. */
unsigned int positionRead(int garbage)
{
	unsigned int pos;
	pos = motorPositionGet(hpGetMotor());
	lastReportedPos[hpGetMotor()] = pos;
	return pos;
}

