/***********************************************************/
/*             (C) Barrett Technology Inc.                 */
/*             139 Main Street                             */
/*             Cambridge, MA 02142, USA                    */
/*             Version 4.0        1998                     */
/*             Author: Daniel Griscom                      */
/*             		   Suitable Systems                    */
/*             		   griscom@suitable.com                */
/*             		   www.suitable.com                    */
/***********************************************************/

/* File: utils.h
	This holds various utilities. */

#ifndef _utils_h
#define _utils_h

/* intEnable enables interrupts. */
#define intEnable()	_asm("cli\n");

/* intDisable disables interrupts. */
#define intDisable()	_asm("sei\n");

/* eeCopy copies data into the EEPROM, from *s2 to *s1. Calls copReset
	regularly. */
void eeCopy(void *s1, const void *s2, unsigned int n);

/* eeVecSet writes an EEPROM interrupt vector. */
void eeVecSet(void *vecLoc, voidFuncVoidPtr funcLoc);

/* copReset resets the Computer Operating Properly timer. */
void copReset(void);

/* strainGaugeInit initializes the strain gauge conversion. */
void strainGaugeInit(void);

/* strainGauge returns the given finger's strain gauge reading. */
int strainGauge(int motor);

/* strainGaugeRead returns the current finger's strain gauge reading. It ignores
	its argument. */
unsigned int strainGaugeRead(int garbage);

/* positionRead returns the current motor's position. It ignores its argument. */
unsigned int positionRead(int garbage);

#endif /* ifndef _utils_h */
